import { Program, Node, Instruction, Runner } from "chaos-toy-rs";
import { batch, Component, createEffect, createSignal, onCleanup } from "solid-js";

const Simulator: Component<{nodes: Node[], instructions: Instruction[]}> = (props) => {
    const [getRunner, setRunner] = createSignal<Runner>();
    const [getWidth, setWidth] = createSignal<number>(100);
    const [getHeight, setHeight] = createSignal<number>(100);
    let canvas: HTMLCanvasElement;
    let parent: HTMLDivElement;
    let ctx: CanvasRenderingContext2D;

    createEffect<Runner>((prev) => {
        if (prev) {
            prev.free();
        }
        // TODO: avoid re-creating a program when nodes and instructions are the same
        const program = Program.from(props.nodes, props.instructions);
        const runner = new Runner(program, getWidth(), getHeight());
        setRunner(runner);

        return runner;
    });

    function resizeListener() {
        batch(() => {
            const dpr = window.devicePixelRatio ?? 1;
            if (parent.clientWidth * dpr !== getWidth()) {
                setWidth(Math.ceil(parent.clientWidth * dpr));
            }
            if (parent.clientHeight * dpr !== getHeight()) {
                setHeight(Math.ceil(parent.clientHeight * dpr));
            }
        });
    }

    function draw() {
        const width = getWidth();
        const height = getHeight();
        ctx.fillStyle = "gray";
        ctx.fillRect(0, 0, width, height);
    }

    createEffect(() => {
        window.addEventListener("resize", resizeListener);
        resizeListener();
        ctx = canvas.getContext("2d") ?? (() => {
            throw new Error("Couldn't create canvas context");
        })();
        draw();

        onCleanup(() => window.removeEventListener("resize", resizeListener));
    });

    return (<div ref={parent!} style={{
        width: "100vw",
        height: "100vh",
        overflow: "hidden",
    }}>
        <canvas
            ref={canvas!}
            width={getWidth()}
            height={getHeight()}
            style={{
                width: "100%",
                height: "100%",
            }}
        >Sorry, your browser needs to support canvases</canvas>
    </div>);
};

export default Simulator;
